var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
import * as React from "react";
import { cloneElement, Component } from "react";
import { matchPath } from "react-router";
import * as PropTypes from "prop-types";
import invariant from "invariant";
import warning from "warning";
import TransitionReplace from "../TransitionReplace";
/**
 * The `<SwitchTransition>` component allows easy integration with `react-router`.
 *
 * Consider the example below using the `Fade` CSS transition from before.
 * As user navigates between routes `SwitchTransition` will detect the first
 * matching route the same way as original `Switch` component does and perform
 * transition between previous and currently matched route using provided transition
 * component. You can use _any_ `<Transition>` component in a `<TransitionReplace>`,
 * not just css.
 *
 * ```jsx
 * import React from 'react';
 * import { Route } from 'react-router';
 * import CSSTransition from 'react-transition-group';
 * import { SwitchTransition } from 'react-transaction-replace';
 *
 * const Fade = ({ children, ...props }) => (
 *     <CSSTransition
 *         { ...props }
 *         timeout={ 300 }
 *         className="fade"
 *     >
 *         { children }
 *     </CSSTransition>
 * );
 *
 * Fade.defaultProps = {
 *     in      : false,
 *     timeout : 300
 * }
 *
 * class RouterExample extends React.Component {
 *     render() {
 *         return (
 *             <SwitchTransition
 *                 transition={ Fade }
 *             >
 *                 <Route path="/foo" component={ Foo } />
 *                 <Route path="/bar" component={ Bar } />
 *             </SwitchTransition>
 *         );
 *     }
 * }
 *
 * ```
 *
 * The `<SwitchTransition>` takes the same props as
 * [`<Switch>`](https://reacttraining.com/react-router/core/api/Switch)
 * or [`<CSSTransition>`](https://reactcommunity.org/react-transition-group/#CSSTransition)
 * or [`<TransitionReplace>`](#TransitionReplace)
 * and additionally accepts the following:
 */
export default class SwitchTransition extends Component {
    componentWillMount() {
        invariant(this.context.router, "You should not use <SwitchTransition> outside a <Router>");
    }
    componentWillReceiveProps(nextProps) {
        warning(!(nextProps.location && !this.props.location), "<Switch> elements should not change from uncontrolled to controlled (or vice versa). You initially used no"
            + "\"location\" prop and then provided one on a subsequent render.");
        warning(!(!nextProps.location && this.props.location), "<SwitchTransition> elements should not change from controlled to uncontrolled (or vice versa). You provided"
            + "a \"location\" prop initially but omitted it on a subsequent render.");
    }
    render() {
        const { route } = this.context.router;
        const _a = this.props, { children, classNames, transition: Transition, timeout, mountOnEnter, unmountOnExit, onExited, onExit, onExiting, onEntered, onEnter, onEntering, addEndListener, location: propLocation } = _a, props = __rest(_a, ["children", "classNames", "transition", "timeout", "mountOnEnter", "unmountOnExit", "onExited", "onExit", "onExiting", "onEntered", "onEnter", "onEntering", "addEndListener", "location"]);
        const location = propLocation || route.location;
        let match = null;
        let child = null;
        React.Children.forEach(children, (element) => {
            if (match === null && React.isValidElement(element)) {
                const { path: pathProp, exact, strict, from } = element.props;
                const path = pathProp || from;
                child = element;
                match = path ? matchPath(location.pathname, { path, exact, strict }) : route.match;
            }
        });
        return (React.createElement(TransitionReplace, Object.assign({ classNames: classNames, timeout: timeout }, props), !match ? null :
            React.createElement(Transition, { key: match ? match.path : route.match.path, timeout: timeout, mountOnEnter: mountOnEnter, unmountOnExit: unmountOnExit, onExited: onExited, onExit: onExit, onExiting: onExiting, onEntered: onEntered, onEnter: onEnter, onEntering: onEntering, addEndListener: addEndListener }, cloneElement(child, { location, computedMatch: match }))));
    }
}
SwitchTransition.contextTypes = {
    router: PropTypes.shape({
        route: PropTypes.object.isRequired
    }).isRequired
};
SwitchTransition.propTypes = {
    /**
     * The transition component used to perform animation.
     * This can be _any_ `<Transition>` component or component that extends it.
     *
     * @type elementType
     */
    transition: PropTypes.func.isRequired
};
//# sourceMappingURL=SwitchTransition.js.map