import invariant from "invariant";
import { Children, cloneElement, isValidElement } from "react";
export function validateChildren(children) {
    const count = Children.count(children);
    invariant(count <= 1, "A <TransitionReplace> may have only one child element");
    if (count === 1) {
        const child = Children.only(children);
        if (!!child.props.children) {
            invariant(!!child.key, "A child of <TransitionReplace> must have unique `key` property set");
        }
    }
}
// use child config unless explicitly set by the TransitionReplace component
function getProp(child, prop, props) {
    return props[prop] !== null ? props[prop] : child.props[prop];
}
export function getChildMapping(key, children, mapFn) {
    const mapper = (child) => (mapFn && isValidElement(child)) ? mapFn(child) : child;
    const result = Object.create(null);
    if (children) {
        const child = Children.only(children);
        result[key] = mapper(child);
    }
    return result;
}
export function getInitialChildMapping(props, handleRef) {
    return getChildMapping("next" /* Next */, props.children, (child) => {
        return cloneElement(child, {
            appear: getProp(child, "appear", props),
            enter: getProp(child, "enter", props),
            exit: getProp(child, "exit", props),
            in: true,
            ref: handleRef.bind(null, "next" /* Next */)
        });
    });
}
export function getNextChildMapping(nextProps, handleRef, prevChildMapping, onExited) {
    const nextChildMapping = getChildMapping("next" /* Next */, nextProps.children, (child) => {
        return cloneElement(child, {
            appear: getProp(child, "appear", nextProps),
            enter: getProp(child, "enter", nextProps),
            exit: getProp(child, "exit", nextProps),
            in: true,
            ref: handleRef.bind(null, "next" /* Next */)
        });
    });
    const prevChild = prevChildMapping["next" /* Next */];
    const nextChild = nextChildMapping["next" /* Next */];
    const previousChildMapping = !isValidElement(prevChild)
        ? null
        : (isValidElement(nextChild) && prevChild.key === nextChild.key
            ? null
            : {
                ["prev" /* Prev */]: cloneElement(prevChild, {
                    onExited: onExited.bind(null, prevChild),
                    enter: getProp(prevChild, "enter", nextProps),
                    exit: getProp(prevChild, "exit", nextProps),
                    in: false,
                    ref: handleRef.bind(null, "prev" /* Prev */)
                })
            });
    return Object.assign({}, previousChildMapping, nextChildMapping);
}
//# sourceMappingURL=ChildMapping.js.map