"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var react_1 = require("react");
var PropTypes = require("prop-types");
var react_dom_1 = require("react-dom");
var PropTypes_1 = require("./utils/PropTypes");
var ChildMapping_1 = require("./utils/ChildMapping");
var DOMUtils_1 = require("./utils/DOMUtils");
var MiscUtils_1 = require("./utils/MiscUtils");
function defaultChildFactory(child) {
    return child;
}
/**
 * The `<TransitionReplace>` component manages a set of `<Transition>` components
 * in a list. Like with the `<Transition>` component, `<TransitionReplace>`, is a
 * state machine for managing the mounting and unmounting of components over
 * time.
 *
 * Consider the example below using the `Fade` CSS transition.
 * As child item changes in the `BasicExample` the `in` prop is toggled
 * automatically by the `<TransitionReplace>`. You can use _any_ `<Transition>`
 * component in a `<TransitionReplace>`, not just css.
 *
 * Note that `<TransitionReplace>`  does not define any animation behavior!
 * Exactly _how_ a list item animates is up to the individual `<Transition>`
 * components. This means you can mix and match animations across different
 * list items.
 *
 * ```jsx
 * import React from 'react';
 * import { Route } from 'react-router';
 * import CSSTransition from 'react-transition-group';
 * import { SwitchTransition } from 'react-transaction-replace';
 *
 * const Fade = ({ children, ...props }) => (
 *     <CSSTransition
 *         { ...props }
 *         timeout={ 300 }
 *         className="fade"
 *     >
 *         { children }
 *     </CSSTransition>
 * );
 *
 * Fade.defaultProps = {
 *     in      : false,
 *     timeout : 300
 * }
 *
 * function random() {
 *     return Math.random().toString(36).substring(7);
 * }
 *
 * class BasicExample extends React.Component {
 *
 *     state = {
 *         key : random()
 *     }
 *
 *     handleClick() {
 *         this.setState({ key : random() })
 *     }
 *
 *     render() {
 *         return (
 *             <div>
 *                 <button onClick={ this.handleClick.bind(this) }>Change</button>
 *
 *                 <TransitionReplace
 *                     transition={ Fade }
 *                     classNames="fade"
 *                 >
 *                     <Fade key={ this.state.key }>
 *                         <h2>Foo</h2>
 *                     </Fade>
 *                 </TransitionReplace>
 *             </div>
 *         );
 *     }
 * }
 * ```
 *
 * In case of `CSSTransition` you probably have some styles defined that for above example looks such as:
 *
 * ```css
 * .fade-exit {
 *     opacity: 1;
 *  }
 *
 * .fade-exit.fade-exit-active {
 *     opacity: 0;
 *     transition: opacity .3s ease-in;
 * }
 *
 * .fade-enter {
 *     opacity: 0;
 * }
 *
 * .fade-enter.fade-enter-active {
 *     opacity: 1;
 *     transition: opacity .3s ease-in;
 * }
 * ```
 *
 * And based on your `classNames` prop the container will also have CSS classes applied so you need
 * to add them to yous stylesheet:
 *
 * ```css
 * .fade-height {
 *     height: auto;
 * }
 *
 * .fade-height-active {
 *     transition: height .3s ease-in-out;
 * }
 *
 * .fade-width {
 *     width: auto;
 * }
 *
 * .fade-width-active {
 *     transition: width .3s ease-in-out;
 * }
 * ```
 *
 * The `<TransitionReplace>` takes the same props as
 * [`<TransitionGroup>`](https://reactcommunity.org/react-transition-group/#TransitionGroup)
 * and additionally accepts the following:
 */
var TransitionReplace = /** @class */ (function (_super) {
    __extends(TransitionReplace, _super);
    function TransitionReplace(props, context) {
        var _this = _super.call(this, props, context) || this;
        _this.mounted = false;
        _this.state = {
            active: false,
            children: null,
            firstRender: true,
            handleExited: _this.handleExited.bind(_this),
            handleRef: _this.handleRef.bind(_this),
            height: null,
            width: null
        };
        return _this;
    }
    TransitionReplace.prototype.getChildContext = function () {
        return {
            transitionGroup: { isMounting: !this.mounted }
        };
    };
    TransitionReplace.prototype.getTransitionDuration = function () {
        var _a = this.state.children, _b = "prev" /* Prev */, prevChild = _a[_b], _c = "next" /* Next */, nextChild = _a[_c];
        return Math.max(
        // Timeout for exit transition
        MiscUtils_1.isNil(prevChild)
            ? 0
            : (typeof prevChild.props.timeout === "number"
                ? prevChild.props.timeout
                : (!MiscUtils_1.isNil(prevChild.props.timeout) ? (prevChild.props.timeout.exit || 0) : 0)), 
        // Timeout for enter transition
        MiscUtils_1.isNil(nextChild)
            ? 0
            : (typeof nextChild.props.timeout === "number"
                ? nextChild.props.timeout
                : (!MiscUtils_1.isNil(nextChild.props.timeout) ? (nextChild.props.timeout.enter || 0) : 0)), 
        // Timeout specified on TransactionReplace
        this.props.timeout);
    };
    TransitionReplace.prototype.handleExited = function (child, node) {
        if (typeof child.props.onExited === "function") {
            child.props.onExited(node);
        }
        this.setState(function (state) {
            var children = __assign({}, state.children);
            delete children["prev" /* Prev */];
            return { children: children };
        });
    };
    TransitionReplace.prototype.handleRef = function (key, instance) {
        switch (key) {
            case "prev" /* Prev */:
                this.refPreviousChild = instance;
                break;
            case "next" /* Next */:
                this.refNextChild = instance;
                break;
        }
    };
    TransitionReplace.getDerivedStateFromProps = function (nextProps, state) {
        ChildMapping_1.validateChildren(nextProps.children);
        var prevChildMapping = state.children, firstRender = state.firstRender, handleExited = state.handleExited, handleRef = state.handleRef;
        return {
            children: firstRender
                ? ChildMapping_1.getInitialChildMapping(nextProps, handleRef)
                : ChildMapping_1.getNextChildMapping(nextProps, handleRef, prevChildMapping, handleExited),
            firstRender: false
        };
    };
    TransitionReplace.prototype.componentWillUnmount = function () {
        clearTimeout(this.timeout);
        cancelAnimationFrame(this.animationId);
    };
    TransitionReplace.prototype.componentDidMount = function () {
        this.mounted = true;
    };
    TransitionReplace.prototype.componentDidUpdate = function (prevProps, prevState) {
        var _this = this;
        var prevNextChild = prevState.children["next" /* Next */];
        var nextChild = this.state.children["next" /* Next */];
        if (react_1.isValidElement(prevNextChild) && react_1.isValidElement(nextChild) && prevNextChild.key === nextChild.key) {
            return;
        }
        var changeWidth = this.props.changeWidth;
        var prevNodeSize = DOMUtils_1.getNodeSize(react_dom_1.findDOMNode(this.refPreviousChild));
        var nextNodeSize = DOMUtils_1.getNodeSize(react_dom_1.findDOMNode(this.refNextChild));
        this.setState(function () { return ({
            active: true,
            height: prevNodeSize.height,
            width: changeWidth ? prevNodeSize.width : null
        }); });
        cancelAnimationFrame(this.animationId);
        this.animationId = requestAnimationFrame(function () {
            _this.setState(function () { return ({
                height: nextNodeSize.height,
                width: changeWidth ? nextNodeSize.width : null
            }); });
            clearTimeout(_this.timeout);
            _this.timeout = setTimeout(function () {
                _this.setState({
                    active: false
                });
            }, _this.getTransitionDuration());
        });
    };
    TransitionReplace.prototype.getClassName = function () {
        var _a = this.props, changeWidth = _a.changeWidth, className = _a.className, classes = _a.classNames;
        var active = this.state.active;
        if (!classes) {
            return className;
        }
        var heightClassName = (typeof classes === "object" && classes !== null)
            ? classes.height || ""
            : classes + "-height";
        var activeHeightClassName = (typeof classes === "object" && classes !== null)
            ? classes.heightActive || ""
            : heightClassName + "-active";
        var widthClassName = (typeof classes === "object" && classes !== null)
            ? classes.width || ""
            : classes + "-width";
        var activeWidthClassName = (typeof classes === "object" && classes !== null)
            ? classes.widthActive || ""
            : widthClassName + "-active";
        return [
            className,
            heightClassName,
            active ? activeHeightClassName : null,
            changeWidth ? widthClassName : null,
            changeWidth && active ? activeWidthClassName : null
        ]
            .filter(function (v) { return !!v; })
            .join(" ");
    };
    TransitionReplace.prototype.render = function () {
        var _a = this.props, changeWidth = _a.changeWidth, _b = _a.childFactory, childFactory = _b === void 0 ? defaultChildFactory : _b, _c = _a.component, Component = _c === void 0 ? "div" : _c, easing = _a.easing, inlineTransitions = _a.inlineTransitions, overflowHidden = _a.overflowHidden, props = __rest(_a, ["changeWidth", "childFactory", "component", "easing", "inlineTransitions", "overflowHidden"]);
        var _d = this.state, active = _d.active, children = _d.children, height = _d.height, width = _d.width;
        var wrapperStyles = {
            height: height,
            overflow: overflowHidden ? "hidden" : "visible",
            position: "relative",
            transitionProperty: "height " + (changeWidth ? ", width " : null),
            transitionTimingFunction: easing,
            transitionDuration: this.getTransitionDuration() + "ms",
            width: changeWidth ? width : null
        };
        delete props.appear;
        delete props.classNames;
        delete props.enter;
        delete props.exit;
        delete props.timeout;
        return (React.createElement(Component, __assign({}, props, { className: this.getClassName(), style: active && inlineTransitions ? wrapperStyles : null }), MiscUtils_1.values(children).map(childFactory)));
    };
    TransitionReplace.propTypes = {
        /**
         * A prop that enables or disables width animations for the container.
         */
        changeWidth: PropTypes.bool,
        /**
         * The animation classNames applied to the component as it enters or exits.
         * A single name can be provided and it will be suffixed for each stage: e.g.
         *
         * `classNames="fade"` applies `fade-height`, `fade-height-active`,
         * `fade-width`, `fade-width-active`.
         * Each individual classNames can also be specified independently like:
         *
         * ```js
         * classNames={{
         *  height: 'my-height',
         *  heightActive: 'my-active-height',
         *  width: 'my-width',
         *  widthActive: 'my-active-width'
         * }}
         * ```
         *
         * @type {string | {
         *  height?: string,
         *  heightActive?: string,
         *  width?: string,
         *  widthActive?: string
         * }}
         */
        classNames: PropTypes_1.classNamesShape,
        /**
         * Defines transition timing function that will be applied to inline CSS styles.
         */
        easing: PropTypes.string,
        /**
         * A prop that enables or disables applying inline CSS styles and transitions to elements.
         */
        inlineTransitions: PropTypes.bool,
        /**
         * A prop that if set to `true` will add `overfolow: hidden` CSS style to the container.
         */
        overflowHidden: PropTypes.bool,
        /**
         * The duration of the transition, in milliseconds.
         * Required to properly apply transition CSS classes and inline styles.
         */
        timeout: PropTypes.number
    };
    TransitionReplace.defaultProps = {
        changeWidth: false,
        easing: "ease",
        inlineTransitions: true,
        overflowHidden: false,
        timeout: 0
    };
    TransitionReplace.childContextTypes = {
        transitionGroup: PropTypes.object.isRequired
    };
    return TransitionReplace;
}(react_1.Component));
exports.default = TransitionReplace;
//# sourceMappingURL=TransitionReplace.js.map