"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var react_1 = require("react");
var react_router_1 = require("react-router");
var PropTypes = require("prop-types");
var invariant_1 = require("invariant");
var warning_1 = require("warning");
var TransitionReplace_1 = require("../TransitionReplace");
/**
 * The `<SwitchTransition>` component allows easy integration with `react-router`.
 *
 * Consider the example below using the `Fade` CSS transition from before.
 * As user navigates between routes `SwitchTransition` will detect the first
 * matching route the same way as original `Switch` component does and perform
 * transition between previous and currently matched route using provided transition
 * component. You can use _any_ `<Transition>` component in a `<TransitionReplace>`,
 * not just css.
 *
 * ```jsx
 * import React from 'react';
 * import { Route } from 'react-router';
 * import CSSTransition from 'react-transition-group';
 * import { SwitchTransition } from 'react-transaction-replace';
 *
 * const Fade = ({ children, ...props }) => (
 *     <CSSTransition
 *         { ...props }
 *         timeout={ 300 }
 *         className="fade"
 *     >
 *         { children }
 *     </CSSTransition>
 * );
 *
 * Fade.defaultProps = {
 *     in      : false,
 *     timeout : 300
 * }
 *
 * class RouterExample extends React.Component {
 *     render() {
 *         return (
 *             <SwitchTransition
 *                 transition={ Fade }
 *             >
 *                 <Route path="/foo" component={ Foo } />
 *                 <Route path="/bar" component={ Bar } />
 *             </SwitchTransition>
 *         );
 *     }
 * }
 *
 * ```
 *
 * The `<SwitchTransition>` takes the same props as
 * [`<Switch>`](https://reacttraining.com/react-router/core/api/Switch)
 * or [`<CSSTransition>`](https://reactcommunity.org/react-transition-group/#CSSTransition)
 * or [`<TransitionReplace>`](#TransitionReplace)
 * and additionally accepts the following:
 */
var SwitchTransition = /** @class */ (function (_super) {
    __extends(SwitchTransition, _super);
    function SwitchTransition() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SwitchTransition.prototype.componentWillMount = function () {
        invariant_1.default(this.context.router, "You should not use <SwitchTransition> outside a <Router>");
    };
    SwitchTransition.prototype.componentWillReceiveProps = function (nextProps) {
        warning_1.default(!(nextProps.location && !this.props.location), "<Switch> elements should not change from uncontrolled to controlled (or vice versa). You initially used no"
            + "\"location\" prop and then provided one on a subsequent render.");
        warning_1.default(!(!nextProps.location && this.props.location), "<SwitchTransition> elements should not change from controlled to uncontrolled (or vice versa). You provided"
            + "a \"location\" prop initially but omitted it on a subsequent render.");
    };
    SwitchTransition.prototype.render = function () {
        var route = this.context.router.route;
        var _a = this.props, children = _a.children, classNames = _a.classNames, Transition = _a.transition, timeout = _a.timeout, mountOnEnter = _a.mountOnEnter, unmountOnExit = _a.unmountOnExit, onExited = _a.onExited, onExit = _a.onExit, onExiting = _a.onExiting, onEntered = _a.onEntered, onEnter = _a.onEnter, onEntering = _a.onEntering, addEndListener = _a.addEndListener, propLocation = _a.location, props = __rest(_a, ["children", "classNames", "transition", "timeout", "mountOnEnter", "unmountOnExit", "onExited", "onExit", "onExiting", "onEntered", "onEnter", "onEntering", "addEndListener", "location"]);
        var location = propLocation || route.location;
        var match = null;
        var child = null;
        React.Children.forEach(children, function (element) {
            if (match === null && React.isValidElement(element)) {
                var _a = element.props, pathProp = _a.path, exact = _a.exact, strict = _a.strict, from = _a.from;
                var path = pathProp || from;
                child = element;
                match = path ? react_router_1.matchPath(location.pathname, { path: path, exact: exact, strict: strict }) : route.match;
            }
        });
        return (React.createElement(TransitionReplace_1.default, __assign({ classNames: classNames, timeout: timeout }, props), !match ? null :
            React.createElement(Transition, { key: match ? match.path : route.match.path, timeout: timeout, mountOnEnter: mountOnEnter, unmountOnExit: unmountOnExit, onExited: onExited, onExit: onExit, onExiting: onExiting, onEntered: onEntered, onEnter: onEnter, onEntering: onEntering, addEndListener: addEndListener }, react_1.cloneElement(child, { location: location, computedMatch: match }))));
    };
    SwitchTransition.contextTypes = {
        router: PropTypes.shape({
            route: PropTypes.object.isRequired
        }).isRequired
    };
    SwitchTransition.propTypes = {
        /**
         * The transition component used to perform animation.
         * This can be _any_ `<Transition>` component or component that extends it.
         *
         * @type elementType
         */
        transition: PropTypes.func.isRequired
    };
    return SwitchTransition;
}(react_1.Component));
exports.default = SwitchTransition;
//# sourceMappingURL=SwitchTransition.js.map