"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var invariant_1 = require("invariant");
var react_1 = require("react");
function validateChildren(children) {
    var count = react_1.Children.count(children);
    invariant_1.default(count <= 1, "A <TransitionReplace> may have only one child element");
    if (count === 1) {
        var child = react_1.Children.only(children);
        if (!!child.props.children) {
            invariant_1.default(!!child.key, "A child of <TransitionReplace> must have unique `key` property set");
        }
    }
}
exports.validateChildren = validateChildren;
// use child config unless explicitly set by the TransitionReplace component
function getProp(child, prop, props) {
    return props[prop] !== null ? props[prop] : child.props[prop];
}
function getChildMapping(key, children, mapFn) {
    var mapper = function (child) { return (mapFn && react_1.isValidElement(child)) ? mapFn(child) : child; };
    var result = Object.create(null);
    if (children) {
        var child = react_1.Children.only(children);
        result[key] = mapper(child);
    }
    return result;
}
exports.getChildMapping = getChildMapping;
function getInitialChildMapping(props, handleRef) {
    return getChildMapping("next" /* Next */, props.children, function (child) {
        return react_1.cloneElement(child, {
            appear: getProp(child, "appear", props),
            enter: getProp(child, "enter", props),
            exit: getProp(child, "exit", props),
            in: true,
            ref: handleRef.bind(null, "next" /* Next */)
        });
    });
}
exports.getInitialChildMapping = getInitialChildMapping;
function getNextChildMapping(nextProps, handleRef, prevChildMapping, onExited) {
    var _a;
    var nextChildMapping = getChildMapping("next" /* Next */, nextProps.children, function (child) {
        return react_1.cloneElement(child, {
            appear: getProp(child, "appear", nextProps),
            enter: getProp(child, "enter", nextProps),
            exit: getProp(child, "exit", nextProps),
            in: true,
            ref: handleRef.bind(null, "next" /* Next */)
        });
    });
    var prevChild = prevChildMapping["next" /* Next */];
    var nextChild = nextChildMapping["next" /* Next */];
    var previousChildMapping = !react_1.isValidElement(prevChild)
        ? null
        : (react_1.isValidElement(nextChild) && prevChild.key === nextChild.key
            ? null
            : (_a = {},
                _a["prev" /* Prev */] = react_1.cloneElement(prevChild, {
                    onExited: onExited.bind(null, prevChild),
                    enter: getProp(prevChild, "enter", nextProps),
                    exit: getProp(prevChild, "exit", nextProps),
                    in: false,
                    ref: handleRef.bind(null, "prev" /* Prev */)
                }),
                _a));
    return __assign({}, previousChildMapping, nextChildMapping);
}
exports.getNextChildMapping = getNextChildMapping;
//# sourceMappingURL=ChildMapping.js.map